 /* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_ITEM_STORE_H__
#define __TK_ITEM_STORE_H__

#include <gtkmm.h>

#include "libtkcommon/tkstring.h"
#include "libtkdb/database.h"
#include "libtkquery/query.h"

#include "editor/item.h"

namespace Tk {
  namespace Editor {
    
    class ItemStoreColumns : public Gtk::TreeModel::ColumnRecord
    {
    public:
      Gtk::TreeModelColumn<Tk::Db::Id> id;
      Gtk::TreeModelColumn<Item *>     item;
      
      ItemStoreColumns() { add(id); add(item); }
    };
    
    class ItemStore : public Gtk::TreeStore
      {
      protected:
	Tk::Db::Database                 *_database;
	ItemStoreColumns                  _columns;
	Tk::String                        _domain;
	
	Gtk::TreeViewColumn::SlotCellData _cell_data_slot;
	
	virtual Item *create_item(const Tk::Db::Id &id) = 0;
	
      private:
	bool check_entity_row(const Gtk::TreeModel::Path &path,
			      const Gtk::TreeModel::iterator &iter,
			      const Tk::Db::Id &entity_id,
			      Gtk::TreeModel::Path &dest_path);
	
      public:
	ItemStore(Tk::Db::Database *database, const Tk::String &domain);
	void refresh();
	void set_entities(const Tk::Db::Ids &ids);
	void on_cell_data(Gtk::CellRenderer *cell, const Gtk::TreeModel::iterator &iter);
	const Gtk::TreeViewColumn::SlotCellData &get_cell_data_slot();
	const Tk::String &get_domain() const;
	Gtk::TreePath find_entity_row(const Tk::Db::Id &entity_id);
	Tk::Db::Id get_row_entity(const Gtk::TreeIter &iter);
      };
    
    
  } /* namespace Editor */
}; /* namespace Tk */

#endif /* __TK_ITEM_STORE_H__ */
