/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "experimenter/database_operation.h"

using namespace Tk::Experimenter;

DatabaseOperation:: DatabaseOperation(Gtk::Window &parent, Tk::Db::Database *database)
  : Gtk::Dialog("Database operation", parent, true)
{
  set_default_size(480, 0);
  database->signal_report().connect( sigc::mem_fun(*this, &DatabaseOperation::report) );
  database->signal_report_update().connect( sigc::mem_fun(*this, &DatabaseOperation::report_update) );
}

Gtk::ProgressBar *
DatabaseOperation:: find_progress(const Tk::Db::Report &report)
{
  ReportProgressMap::iterator iter( _reports.find(&report) );
  
  if (iter == _reports.end())
    return 0;
  
  return iter->second;
}

void
DatabaseOperation:: report(const Tk::Db::Report &report)
{
  /*
  WDEBUG_OUT << L"Received report with description `"
	     << report.get_description()
	     << L"'"
	     << std::endl;
  */
  
  Gtk::ProgressBar *progress( new Gtk::ProgressBar() );
  progress->set_text(report.get_description());
  progress->set_fraction(0.0);
  
  get_vbox()->pack_start(*progress, true, true);
  progress->show();
  
  _reports.insert( ReportProgressMap::value_type(&report, progress) );
}

void
DatabaseOperation:: report_update(const Tk::Db::ReportUpdate &update)
{
  /*
  WDEBUG_OUT << L"Received report update with fraction of `"
	     << update.get_fraction()
	     << L"' for report with description `"
	     << update.get_report().get_description()
	     << L"'"
	     << std::endl;
  */
  
  const Tk::Db::Report &report( update.get_report() );
  Gtk::ProgressBar *progress( find_progress(report) );
  if (!progress)
    throw std::logic_error(std::string("No progress bar for report with description `")
			   + static_cast<std::string>(report.get_description())
			   + "'");
  
  if (update.get_status() == Tk::Db::ReportUpdate::Running)
    progress->set_fraction(update.get_fraction());
  else
    {
      get_vbox()->remove(*progress);
      delete progress;
      _reports.erase(&report);
      
      if (_reports.empty())
	response(Gtk::RESPONSE_NONE);
    }
}
