/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_EXP_PARAMETERS_H__
#define __TK_EXP_PARAMETERS_H__

#include <gtkmm.h>

#include "libtkcommon/tkstring.h"
#include "libtkdb/database.h"
#include "experimenter/screen_size.h"
#include "experimenter/randomness_source.h"
#include "experimenter/bit_multiplier.h"
#include "experimenter/algorithm_spec.h"
#include "experimenter/trace_count.h"
#include "experimenter/total_time.h"
#include "experimenter/speed.h"
#include "experimenter/horizon.h"

namespace Tk {
  namespace Experimenter {

    class Parameters : public Gtk::Frame
      {
      private:
	AlgorithmSpec             _algorithm_spec;
	BitMultiplier             _bit_multiplier;
	TraceCount                _trace_count;
	TotalTime                 _total_time;
	Speed                     _speed;
	Horizon                   _horizon;
	ScreenSize                _screen_size;
	RandomnessSource          _randomness;
      public:
	Parameters(Db::Database *database);
	String get_algorithm_spec() const;
	unsigned get_bit_multiplier() const;
	unsigned get_trace_count() const;
	unsigned get_total_time() const;
	bool get_scrolled() const;
	double get_speed() const;
	double get_horizon() const;
	unsigned get_width() const;
	unsigned get_height() const;
	String get_retriever_type() const;
      };

  } /* namespace Experimenter */
}; /* namespace Tk */

#endif /* __TK_EXP_PARAMETERS_H__ */
