/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <iostream>
#include <fstream>

#include <odbc++/types.h>

#define TK_STREAM_DEBUG
#include "libtkcommon/stream.h"

template <typename T>
static void
print_char_type_details()
{
  std::cout << "digits: "
	    << std::numeric_limits<T>::digits
	    << "; signed: "
	    << std::numeric_limits<T>::is_signed
	    << "; radix: "
	    << std::numeric_limits<T>::radix
	    << std::endl;
}

int main (int argc, char **argv)
{
  if (argc != 2)
    {
      std::cout << "signed char   ";
      print_char_type_details<signed char>();
      
      std::cout << "unsigned char ";
      print_char_type_details<unsigned char>();
      
      std::cout << "char          ";
      print_char_type_details<char>();
      
      std::cout << "wchar_t       ";
      print_char_type_details<wchar_t>();
      
      return 0;
    }
  
  std::ifstream file(argv[1]);
  
  Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE> *odbc_strm( new Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE>(file) );
  
  Tk::BasicStream<wchar_t> *wchar_strm( odbc_strm->write_stream<wchar_t>() );
  delete odbc_strm;

  Tk::BasicStream<unsigned char> *uchar_strm( wchar_strm->write_stream<unsigned char>() );
  delete wchar_strm;
  
  odbc_strm = uchar_strm->write_stream<ODBCXX_SIGNED_CHAR_TYPE>();
  delete uchar_strm;
 
  Tk::BasicStream<long long> *d_strm(odbc_strm->write_stream<long long>());
  delete odbc_strm;

  Tk::Stream *std_strm(d_strm->write_stream<char>());
  delete d_strm;

  char c;
  while (std_strm->get_stream()->read(&c, 1))
    std::cout << static_cast<char>(c);
  std::cout.flush();
  
  delete std_strm;
  
  return 0;
}
