/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#define TK_STREAM_DEBUG
#include "libtkcommon/stream.h"
#include "libtkrandom/randomness.h"

using namespace Tk::Random;

void
print_line(std::ostringstream **bits_line_ptr, std::ostringstream **stream_line_ptr, unsigned line_count)
{
  std::cout << "Bits line\t" << line_count << ": " << (*bits_line_ptr)->str() << std::endl;
  std::cout << "Stream line\t" << line_count << ": " << (*stream_line_ptr)->str() << std::endl;
  
  delete *bits_line_ptr;
  delete *stream_line_ptr;
  *bits_line_ptr = new std::ostringstream;
  *stream_line_ptr = new std::ostringstream;
}

int main (int argc, char **argv)
{
  typedef unsigned char conv_char_type;
  const unsigned short BITS_PER_CONV_CHAR =
    std::numeric_limits<conv_char_type>::digits + static_cast<unsigned>(std::numeric_limits<conv_char_type>::is_signed);
  
  Randomness rand("pseudo", false, 64, true);
  std::wcout.flush();
  for (unsigned i(0); i < 800; ++i)
    rand.get_bit();
  Bits bits(rand.get_used_bits());
  
  BitStream *bitchar_stream(bits.get_data());
  Tk::BasicStream<wchar_t> *wchar_stream(bitchar_stream->write_stream<wchar_t>());
  delete bitchar_stream;
  Tk::BasicStream<char> *char_stream(wchar_stream->write_stream<char>());
  delete wchar_stream;
  Tk::BasicStream<conv_char_type> *conv_stream(char_stream->write_stream<conv_char_type>());
  delete char_stream;

  conv_char_type c;
  const unsigned line_len(80);
  unsigned out_count(0);
  unsigned line_count(0);
  std::ostringstream *bits_line( new std::ostringstream );
  std::ostringstream *stream_line( new std::ostringstream );
  
  Bits::iterator bits_iter(bits.begin());
  std::cout << "Total bits: " << bits.size() << std::endl;
  std::cout << "Total chars: " << char_stream->get_len() << std::endl;
  while (conv_stream->get_stream()->get(c))
    {
      for (unsigned i(0); i < BITS_PER_CONV_CHAR; ++i, ++bits_iter)
	{
	  (*bits_line) << (*bits_iter ? "1" : "0");
	  (*stream_line) << ((c & (1 << (BITS_PER_CONV_CHAR - i - 1))) ? "1" : "0");
	  
	  if (out_count == line_len - 1)
	    {
	      print_line(&bits_line, &stream_line, line_count);
	      out_count = 0;
	      ++line_count;
	    }
	  else
	    ++out_count;
	}
      
      if (out_count > 0)
	{
	 (*bits_line) << ' ';
	 (*stream_line) << ' ';
	}
    }
  
  std::cout.flush();
  
  delete bits_line;
  delete stream_line;
  delete conv_stream;
}
