/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_RANDOMNESS_H__
#define __TK_RANDOMNESS_H__

#include <vector>

#include "libtkcommon/tkstring.h"
#include "random_types.h"
#include "retriever_registry.h"
#include "retriever.h"

namespace Tk {

  namespace Random {

    class Randomness
      {
      private:
      typedef std::list<Retriever *> Retrievers;
      typedef enum { Stopped, Running } RunState;

      pthread_t         _join_thread;
      pthread_mutex_t   _join_lock; 
      pthread_cond_t    _join_signal;
      RunState          _run_state;
      String            _retriever_type;
      Retrievers        _retrievers;
      unsigned          _retrieving;
      unsigned          _retrieved;
      
      Buffer            _buffer;
      unsigned          _buffer_size;
      unsigned          _threshold; /**< In bytes (chars) >*/
      
      char              _randomness;
      unsigned short    _bit_index;
      
      bool              _deterministic;
      unsigned          _needed;
      
      Bits            * _used_bits;

      static void * low_join_run (void *);
      void * join_run ();

      void lock ();
      void unlock ();

      void append_buffer (const Buffer * buf);
      void retrieve (unsigned count);
      unsigned retrieved();
      RunState run_state();
      void check_threshold ();

    public:
      /// Generate a random number between 0 and upper
      static unsigned long get_number(unsigned long upper, const String & retriever_type = "alea");
      
      Randomness(const String &retriever_type, bool deterministic, unsigned buffer_size, bool record_use = true);
      ~Randomness();

      bool get_bit();
      const Bits& get_used_bits() const;

    };
    
  }; /* namespace Random */
  
}; /* namespace Tk */


#endif /* __TK_RANDOMNESS_H__ */
